package com.iflytek.cyber.home.sdk.demo

import android.Manifest
import android.app.ProgressDialog
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.net.ConnectivityManager
import android.net.wifi.WifiConfiguration
import android.net.wifi.WifiManager
import android.os.Build
import android.os.Bundle
import android.os.IBinder
import android.view.MenuItem
import android.view.View
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.PermissionChecker
import com.google.android.material.textfield.TextInputLayout
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.ResponseCallback
import kotlinx.android.synthetic.main.activity_ap_config_net.*
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Response
import java.net.Socket
import java.text.SimpleDateFormat
import java.util.*

class ApConfigNetActivity : AppCompatActivity() {
    private var apConfigNetService: ApConfigNetService? = null
    private val serviceConnection = object : ServiceConnection {
        override fun onServiceDisconnected(name: ComponentName?) {
            apConfigNetService?.removeListener(socketListener)
            apConfigNetService = null
        }

        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            (service as? ApConfigNetService.ApServiceBinder)?.let {
                apConfigNetService = it.service

                apConfigNetService?.addListener(socketListener)

                val gateway = apConfigNetService?.getCurrentGateway()
                current_gateway.text = "当前网关: $gateway"
            }
        }

    }

    private val socketListener = object : ApConfigNetService.SocketListener() {
        override fun onOpen(socket: Socket) {
            appendLog("$socket 已连接")
        }

        override fun onClosed(socket: Socket, t: Throwable?) {
            appendLog(
                """
                $socket 已断开
                $t
            """.trimIndent()
            )
        }

        override fun onMessage(socket: Socket, byteArray: ByteArray) {
            val useBytes = message_at_bytes.isChecked
            if (useBytes) {
                appendLog(
                    """
                    接收信息:
                    ${byteArray.contentToString()}
                """.trimIndent()
                )
            } else {
                val string = String(byteArray)
                appendLog(
                    """
                    接收信息:
                    $string
                """.trimIndent()
                )
            }
        }

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_ap_config_net)

        connect.setOnClickListener {
            apConfigNetService?.connect() ?: run {
                startService(Intent(this, ApConfigNetService::class.java).apply {
                    action = ApConfigNetService.ACTION_CONNECT
                })
            }
        }
        disconnect.setOnClickListener {
            apConfigNetService?.let { service ->
                if (service.isConnected() == true) {
                    service.disconnect()
                } else {
                    appendLog("Socket 未连接")
                }
            } ?: run {
                startService(Intent(this, ApConfigNetService::class.java).apply {
                    action = ApConfigNetService.ACTION_DISCONNECT
                })
            }
        }
        send_config.setOnClickListener {
            val ssid = wifi_ssid.text.toString()
            val bssid = wifi_bssid.text.toString()
            val password = wifi_password.text.toString()
            val code = auth_code.text.toString()

            val json = JSONObject().apply {
                put("ssid", ssid)
                if (bssid.isNotEmpty()) {
                    put("bssid", bssid)
                }
                if (password.isNotEmpty()) {
                    put("password", password)
                }
                put("auth_code", code)
            }
            apConfigNetService?.let { service ->
                if (service.isConnected() == true) {
                    appendLog("尝试发送: \n$json")
                    service.send(json.toString())
                } else {
                    appendLog("Socket 未连接")
                }
            } ?: run {
                appendLog("尝试发送: \n$json")
                startService(Intent(this, ApConfigNetService::class.java).apply {
                    action = ApConfigNetService.ACTION_SEND_MESSAGE
                    putExtra(ApConfigNetService.EXTRA_MESSAGE, json.toString())
                })
            }
        }
        send_timestamp.setOnClickListener {
            val json = JSONObject().apply {
                put("timestamp", System.currentTimeMillis())
            }
            apConfigNetService?.let { service ->
                if (service.isConnected() == true) {
                    appendLog("尝试发送: $json")
                    service.send(json.toString())
                } else {
                    appendLog("Socket 未连接")
                }
            } ?: run {
                startService(Intent(this, ApConfigNetService::class.java).apply {
                    action = ApConfigNetService.ACTION_SEND_MESSAGE
                    putExtra(ApConfigNetService.EXTRA_MESSAGE, json.toString())
                })
            }
        }
        get_current_wifi_info.setOnClickListener {
            val context = it.context
            if (PermissionChecker.checkSelfPermission(
                    context,
                    Manifest.permission.ACCESS_FINE_LOCATION
                ) == PermissionChecker.PERMISSION_GRANTED
            ) {
                wifi_bssid.setText(getConnectedBssid(context))
                wifi_ssid.setText(getConnectedSsid(context))
            } else {
                ActivityCompat.requestPermissions(
                    this,
                    arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                    10000
                )
            }
        }

        check_auth_code.setOnClickListener {
            val context = it.context
            val authCode = auth_code.text.toString()
            if (authCode.isEmpty()) {
                Toast.makeText(context, "请先获取 auth_code", Toast.LENGTH_SHORT).show()
            } else {
                IFlyHome.checkAuthCodeState(authCode, object : ResponseCallback {
                    override fun onResponse(response: Response<String>) {
                        appendLog(
                            if (response.isSuccessful) {
                                response.body() ?: ""
                            } else {
                                response.errorBody()?.string() ?: ""
                            }
                        )
                    }

                    override fun onFailure(call: Call<String>, t: Throwable) {
                        appendLog(t.toString())
                    }

                })
            }
        }
        get_auth_code.setOnClickListener {
            val context = it.context
            val layout = FrameLayout(context)
            val editText = EditText(context)
            editText.hint = "client_id"
            layout.addView(
                editText,
                FrameLayout.LayoutParams(
                    FrameLayout.LayoutParams.MATCH_PARENT,
                    FrameLayout.LayoutParams.WRAP_CONTENT
                ).apply {
                    leftMargin = 40
                    rightMargin = 40
                }
            )
            AlertDialog.Builder(context)
                .setView(layout)
                .setTitle("生成授权码")
                .setPositiveButton(android.R.string.yes) { _, _ ->
                    val clientId = editText.text.toString()
                    if (clientId.isNotEmpty()) {
                        getAuthCode(clientId)
                    } else {
                        Toast.makeText(context, "请先输入 client_id", Toast.LENGTH_SHORT).show()
                    }
                }
                .setNegativeButton(android.R.string.no, null)
                .show()
        }

        val intent = Intent(this, ApConfigNetService::class.java)
        bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE)

        supportActionBar?.let {
            it.setHomeButtonEnabled(true)
            it.setDisplayHomeAsUpEnabled(true)
        }
    }

    private fun getAuthCode(clientId: String) {
        val progressDialog = ProgressDialog(this)
        progressDialog.setTitle("请求授权码")
        progressDialog.setMessage("请稍候")
        progressDialog.show()

        IFlyHome.getClientAuthCode(clientId, null, object : ResponseCallback {
            override fun onResponse(response: Response<String>) {
                progressDialog.dismiss()
                if (response.isSuccessful) {
                    response.body()?.let { body ->
                        val json = JSONObject(body)
                        if (json.has("auth_code")) {
                            auth_code.setText(json.getString("auth_code"))
                        }
                        appendLog(
                            """
                            请求授权码:
                            $body
                        """.trimIndent()
                        )
                    } ?: run {

                    }
                } else {
                    response.errorBody()?.string()?.let {

                    }
                }
            }

            override fun onFailure(call: Call<String>, t: Throwable) {
                progressDialog.dismiss()
                t.printStackTrace()
            }
        })
    }

    override fun onResume() {
        super.onResume()

        apConfigNetService?.let { service ->
            val gateway = service.getCurrentGateway()
            current_gateway.text = "当前网关: $gateway"
        }
    }

    override fun onDestroy() {
        super.onDestroy()

        unbindService(serviceConnection)
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home) {
            onBackPressed()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    private fun getConnectedSsid(context: Context?): String? {
        (context?.applicationContext?.getSystemService(Context.WIFI_SERVICE)
            as? WifiManager)?.let {
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.P) {
                val configuredNetworks = it.configuredNetworks
                if (configuredNetworks?.isNotEmpty() == true)
                    for (config in configuredNetworks) {
                        val ssid = config.SSID.substring(1, config.SSID.length - 1)

                        if (config.status == WifiConfiguration.Status.CURRENT) {
                            return ssid
                        }
                    }
            } else {
                val connManager = context.applicationContext
                    .getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
                val networkInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI)
                if (networkInfo.isConnected) {
                    val wifiInfo = it.connectionInfo
                    val ssid = wifiInfo.ssid
                    return ssid.substring(1, ssid.length - 1)
                }
            }
        }
        return null
    }

    private fun getConnectedBssid(context: Context?): String? {
        (context?.applicationContext?.getSystemService(Context.WIFI_SERVICE)
            as? WifiManager)?.let {
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.P) {
                val configuredNetworks = it.configuredNetworks
                if (configuredNetworks?.isNotEmpty() == true)
                    for (config in configuredNetworks) {
                        val bssid = config.BSSID

                        if (config.status == WifiConfiguration.Status.CURRENT) {
                            return bssid
                        }
                    }
            } else {
                val connManager = context.applicationContext
                    .getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
                val networkInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI)
                if (networkInfo.isConnected) {
                    val wifiInfo = it.connectionInfo
                    return wifiInfo.bssid
                }
            }
        }
        return null
    }

    private fun appendLog(logMessage: String) {
        val currentLog = log.text
        val stringBuilder = StringBuilder(currentLog)
        stringBuilder.append(SimpleDateFormat("mm:ss.SS", Locale.getDefault()).format(Date()))
        stringBuilder.append("  ")
        stringBuilder.append(logMessage)
        stringBuilder.append("\n")
        runOnUiThread {
            log.text = stringBuilder.toString()

            log_container.post {
                log_container.smoothScrollTo(0, log.height)
            }
        }
    }
}