package com.iflytek.cyber.home.sdk.demo

import android.app.ProgressDialog
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.DialogFragment
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.Snackbar
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.ResponseCallback
import com.iflytek.home.sdk.model.CollectionBody
import kotlinx.android.synthetic.main.activity_api.*
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Response

class ApiActivity : AppCompatActivity() {

    private var progressDialog: ProgressDialog? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_api)

        supportActionBar?.run {
            setHomeButtonEnabled(true)
            setDisplayHomeAsUpEnabled(true)
        }

        progressDialog = ProgressDialog(this)

        api_list.layoutManager = GridLayoutManager(this, 3)
        api_list.adapter = ApiAdapter()
    }

    inner class ApiAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        private val apiName = resources.getStringArray(R.array.api_array)

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = ApiActivity.ApiViewHolder(
                LayoutInflater.from(this@ApiActivity).inflate(
                    R.layout.item_api,
                    parent,
                    false
                )
            )
            holder.itemView.setOnClickListener {
                onApiClick(holder.adapterPosition)
            }
            return holder
        }

        override fun getItemCount(): Int {
            return apiName.size
        }

        override fun onBindViewHolder(viewHolder: RecyclerView.ViewHolder, position: Int) {
            if (viewHolder is ApiActivity.ApiViewHolder) {
                viewHolder.tvTitle.text = apiName[position]
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home) {
            onBackPressed()
        }
        return super.onOptionsItemSelected(item)
    }

    @Suppress("DEPRECATION")
    private fun onApiClick(position: Int) {
        var result = IFlyHome.RESULT_OK
        Log.d("Api", "position: $position")
        when (resources.getStringArray(R.array.api_array)[position]) {
            getString(R.string.get_user_devices) -> {
                progressDialog = ProgressDialog(this)
                progressDialog?.setMessage("请稍候")
                progressDialog?.show()
                result = IFlyHome.getUserDevices(responseCallback())
            }
            getString(R.string.get_device_info) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.getDeviceDetail(deviceId, responseCallback())
                }
            }
            getString(R.string.delete_user_device) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.deleteUserDevice(deviceId, responseCallback())
                }
            }
            getString(R.string.get_music_groups) -> {
                var deviceId: String? = device_id.text.toString()
                if (deviceId.isNullOrEmpty()) {
                    deviceId = null
                }
                result = IFlyHome.getMusicGroups(deviceId, responseCallback())
            }
            getString(R.string.search_music) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("search", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        var deviceId: String? = device_id.text.toString()
                        val search = map["search"]
                        if (deviceId.isNullOrEmpty()) {
                            deviceId = null
                        }
                        if (search.isNullOrEmpty()) {
                            dialog.dismiss()
                            showAlert("搜索关键字不能为空")
                        } else {
                            val requestResult = IFlyHome.searchMusic(
                                deviceId, search, 0, 10, responseCallback(dialog)
                            )
                            handleResult(requestResult)
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.get_music_control_state) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不能为空")
                } else {
                    result = IFlyHome.getMusicControlState(deviceId, responseCallback())
                }
            }
            getString(R.string.get_section) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("sectionId", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val sectionId = map["sectionId"]
                        when {
                            sectionId.isNullOrEmpty() -> {
                                dialog.dismiss()
                                showAlert("sectionId 不能为空")
                            }
                            device_id.text.isNullOrEmpty() -> {
                                dialog.dismiss()
                                showAlert("deviceId 不能为空")
                            }
                            else -> {
                                val requestResult = IFlyHome.getGroupSection(
                                    sectionId, device_id.text.toString(),
                                    responseCallback(dialog)
                                )
                                handleResult(requestResult)
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.music_control_play_collections) -> {
                var mediaId: String? = media_id.text.toString()
                val deviceId = device_id.text.toString()
                val sourceType = source_type.text.toString()
                when {
                    deviceId.isEmpty() -> {
                        showAlert("deviceId 不能为空")
                    }
                    else -> {
                        val dialog = RequestDialog()
                        val params = Bundle()
                        params.putString("tagId", "")
                        dialog.arguments = params
                        dialog.handler = object : RequestDialog.RequestHandler {
                            override fun handle(map: Map<String, String>) {
                                try {
                                    val tagId = map["tagId"]?.toInt() ?: 0
                                    if (mediaId.isNullOrEmpty())
                                        mediaId = null
                                    result = IFlyHome.musicControlPlayCollections(
                                        tagId, deviceId, mediaId, sourceType, responseCallback()
                                    )
                                } catch (nfe: NumberFormatException) {
                                    nfe.printStackTrace()
                                    dialog.restoreProgress()
                                    showAlert("tagId 不符合要求")
                                }
                            }
                        }
                        dialog.show(supportFragmentManager, "请求")
                    }
                }
            }
            getString(R.string.music_control_play) -> {
                val mediaId = media_id.text.toString()
                val deviceId = device_id.text.toString()
                val sourceType = source_type.text.toString()
                when {
                    deviceId.isEmpty() -> {
                        showAlert("deviceId 不能为空")
                    }
                    mediaId.isEmpty() -> {
                        showAlert("mediaId 不能为空")
                    }
                    sourceType.isEmpty() -> {
                        showAlert("sourceType 不能为空")
                    }
                    else -> {
                        result = IFlyHome.musicControlPlay(
                            deviceId, mediaId, sourceType, responseCallback()
                        )
                    }
                }
            }
            getString(R.string.music_control_play_group) -> {
                var mediaId: String? = media_id.text.toString()
                val groupId = group_id.text.toString()
                val deviceId = device_id.text.toString()
                if (mediaId.isNullOrEmpty()) mediaId = null
                when {
                    deviceId.isEmpty() -> {
                        showAlert("deviceId 不能为空")
                    }
                    groupId.isEmpty() -> {
                        showAlert("groupId 不能为空")
                    }
                    else -> {
                        result = IFlyHome.musicControlPlayGroup(
                            deviceId, mediaId, groupId, responseCallback()
                        )
                    }
                }
            }
            getString(R.string.unlike_source) -> {
                val sourceType = source_type.text.toString()
                if (sourceType.isEmpty()) {
                    showAlert("sourceType 不可为空")
                } else {
                    result = IFlyHome.unlikeSource(sourceType, responseCallback())
                }
            }
            getString(R.string.infrared_devices_add) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("extends", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val deviceId = device_id.text.toString()
                        val extends = map["extends"]
                        val jsonObject: JSONObject?
                        try {
                            jsonObject = JSONObject(extends)
                        } catch (e: Exception) {
                            dialog.restoreProgress()
                            showAlert("extends 不符合 json 格式")
                            return
                        }
                        when {
                            deviceId.isEmpty() -> {
                                dialog.restoreProgress()
                                showAlert("deviceId 不可为空")
                            }
                            else -> {
                                val requestResult = IFlyHome.infraredDevicesAdd(
                                    deviceId,
                                    jsonObject,
                                    responseCallback(dialog)
                                )
                                handleResult(requestResult)
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.infrared_devices_accept) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("extends", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val deviceId = device_id.text.toString()
                        val extends = map["extends"]
                        val jsonObject: JSONObject?
                        try {
                            jsonObject = JSONObject(extends)
                        } catch (e: Exception) {
                            dialog.restoreProgress()
                            showAlert("extends 不符合 json 格式")
                            return
                        }
                        when {
                            deviceId.isEmpty() -> {
                                dialog.restoreProgress()
                                showAlert("deviceId 不可为空")
                            }
                            else -> {
                                val requestResult = IFlyHome.infraredDevicesAccept(
                                    deviceId,
                                    jsonObject,
                                    responseCallback(dialog)
                                )
                                handleResult(requestResult)
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.music_control_stop) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.musicControlStop(deviceId, responseCallback())
                }
            }
            getString(R.string.music_control_resume) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.musicControlResume(deviceId, responseCallback())
                }
            }
            getString(R.string.music_control_previous) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.musicControlPrevious(deviceId, responseCallback())
                }
            }
            getString(R.string.music_control_next) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.musicControlNext(deviceId, responseCallback())
                }
            }
            getString(R.string.like_music) -> {
                val mediaId = media_id.text.toString()
                val sourceType = source_type.text.toString()
                when {
                    mediaId.isEmpty() -> showAlert("mediaId 不可为空")
                    sourceType.isEmpty() -> showAlert("sourceType 不可为空")
                    else -> {
                        result = IFlyHome.likeMusic(
                            mediaId, sourceType, responseCallback()
                        )
                    }
                }
            }
            getString(R.string.music_collected) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("likeType", "")
                params.putString("mediaId", "")
                params.putString("sourceType", "")
                params.putString("business", "")
                params.putString("tagId", "")
                params.putString("albumId", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val likeType = map["likeType"]
                        val mediaId = map["mediaId"]
                        val sourceType = map["sourceType"]
                        val business = map["business"]
                        val tagId = map["tagId"]
                        val albumId = map["albumId"]
                        when {
                            likeType.isNullOrEmpty() || (likeType != "single" && likeType != "album") -> {
                                showAlert("likeType 必须为 album 或 single")
                                dialog.restoreProgress()
                            }
                            tagId.isNullOrEmpty() -> {
                                showAlert("tagId 不能为空")
                                dialog.restoreProgress()
                            }
                            else -> {
                                try {
                                    val tagIdInt = tagId.toInt()
                                    val collectionBody = CollectionBody(
                                        likeType,
                                        mediaId,
                                        sourceType,
                                        business,
                                        tagIdInt,
                                        albumId,
                                        null
                                    )
                                    IFlyHome.musicCollected(collectionBody, responseCallback())
                                    dialog.dismiss()
                                } catch (nfe: NumberFormatException) {
                                    nfe.printStackTrace()
                                    dialog.restoreProgress()
                                    showAlert("tagId 不符合要求")
                                }
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.delete_collected) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("likeType", "")
                params.putString("mediaId", "")
                params.putString("tagId", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val likeType = map["likeType"]
                        val mediaId = map["mediaId"]
                        val tagId = map["tagId"]
                        when {
                            likeType.isNullOrEmpty() || (likeType != "single" && likeType != "album") -> {
                                showAlert("likeType 必须为 album 或 single")
                                dialog.restoreProgress()
                            }
                            mediaId.isNullOrEmpty() -> {
                                showAlert("mediaId 不能为空")
                                dialog.restoreProgress()
                            }
                            tagId.isNullOrEmpty() -> {
                                showAlert("tagId 不能为空")
                                dialog.restoreProgress()
                            }
                            else -> {
                                try {
                                    val tagIdInt = tagId.toInt()
                                    val collectionBody = CollectionBody(
                                        likeType,
                                        mediaId,
                                        null,
                                        null,
                                        tagIdInt,
                                        null,
                                        arrayListOf(mediaId)
                                    )
                                    IFlyHome.deleteCollected(collectionBody, responseCallback())
                                    dialog.dismiss()
                                } catch (nfe: NumberFormatException) {
                                    nfe.printStackTrace()
                                    dialog.restoreProgress()
                                    showAlert("tagId 不符合要求")
                                }
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.sync_collection) -> {
                IFlyHome.asyncCollection(responseCallback())
            }
            getString(R.string.get_album) -> {
                val deviceId = device_id.text.toString()
                when {
                    deviceId.isEmpty() ->
                        showAlert("deviceId 不可为空")
                    else -> {
                        val dialog = RequestDialog()
                        val params = Bundle()
                        params.putString("albumId", "")
                        params.putString("sourceType", "")
                        params.putString("business", "")
                        dialog.arguments = params
                        dialog.handler = object : RequestDialog.RequestHandler {
                            override fun handle(map: Map<String, String>) {
                                val albumId = map["albumId"]
                                val sourceType = map["sourceType"]
                                val business = map["business"]
                                when {
                                    albumId.isNullOrEmpty() -> {
                                        showAlert("albumId 不能为空")
                                        dialog.restoreProgress()
                                    }
                                    sourceType.isNullOrEmpty() -> {
                                        showAlert("sourceType 不能为空")
                                        dialog.restoreProgress()
                                    }
                                    business.isNullOrEmpty() -> {
                                        showAlert("business 不能为空")
                                        dialog.restoreProgress()
                                    }
                                    else -> {
                                        result = IFlyHome.getAlbum(
                                            albumId,
                                            deviceId,
                                            sourceType,
                                            business,
                                            1,
                                            10,
                                            responseCallback()
                                        )
                                        dialog.dismiss()
                                    }
                                }
                            }
                        }
                        dialog.show(supportFragmentManager, "请求")
                    }
                }
            }
            getString(R.string.get_music_collections) -> {
                val deviceId = device_id.text.toString()
                val sourceType = source_type.text.toString()
                when {
                    sourceType.isEmpty() ->
                        showAlert("sourceType 不可为空")
                    else -> {
                        result = IFlyHome.getCollectionFromSource(
                            deviceId, sourceType, responseCallback()
                        )
                    }
                }
            }
            getString(R.string.get_music_collection_source) -> {
                result = IFlyHome.getCollectionSource(responseCallback())
            }
            getString(R.string.get_music_collection_tag) -> {
                result = IFlyHome.getCollectionTags(responseCallback())
            }
            getString(R.string.get_music_collections_from_tag) -> {
                val deviceId = device_id.text.toString()
                when {
                    deviceId.isEmpty() ->
                        showAlert("deviceId 不可为空")
                    else -> {
                        val dialog = RequestDialog()
                        val params = Bundle()
                        params.putString("tagId", "")
                        dialog.arguments = params
                        dialog.handler = object : RequestDialog.RequestHandler {
                            override fun handle(map: Map<String, String>) {
                                try {
                                    val tagId = map["tagId"]?.toInt() ?: 0
                                    result = IFlyHome.getSongCollection(
                                        deviceId, tagId, responseCallback()
                                    )
                                    dialog.dismiss()
                                } catch (nfe: NumberFormatException) {
                                    nfe.printStackTrace()
                                    dialog.restoreProgress()
                                    showAlert("tagId 不符合要求")
                                }
                            }
                        }
                        dialog.show(supportFragmentManager, "请求")
                    }
                }
            }
            getString(R.string.unlike_music) -> {
                val mediaId = media_id.text.toString()
                val sourceType = source_type.text.toString()
                if (mediaId.isEmpty()) {
                    showAlert("mediaId 不可为空")
                } else
                    result = IFlyHome.unlikeMusic(arrayOf(mediaId), sourceType, responseCallback())
            }
            getString(R.string.music_control_set_volume) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("volume", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        val deviceId = device_id.text.toString()
                        val volume = map["volume"]
                        var volumeInt = -1
                        try {
                            volumeInt = volume?.toInt() ?: -1
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                        when {
                            deviceId.isEmpty() -> {
                                dialog.dismiss()
                                showAlert("deviceId 不可为空")
                            }
                            volumeInt == -1 -> {
                                dialog.dismiss()
                                showAlert("volume 不合法")
                            }
                            else -> {
                                val requestResult = IFlyHome.musicControlSetVolume(
                                    deviceId,
                                    volumeInt,
                                    responseCallback(dialog)
                                )
                                handleResult(requestResult)
                            }
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.reset_network) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.resetNetwork(deviceId, responseCallback())
                }
            }
            getString(R.string.restore_factory) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.restoreFactory(deviceId, responseCallback())
                }
            }
            getString(R.string.reboot_device) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.rebootDevice(deviceId, responseCallback())
                }
            }
            getString(R.string.get_songs) -> {
                val groupId = group_id.text.toString()
                if (groupId.isEmpty()) {
                    showAlert("groupId 不可为空")
                } else {
                    result = IFlyHome.getSongs(groupId, 1, 10, responseCallback())
                }
            }
            getString(R.string.get_user_id) -> {
                result = IFlyHome.getUserIdentify(responseCallback())
            }
            getString(R.string.get_device_code) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("client_id", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            val clientId = map["client_id"]
                            dialog.dismiss()
                            if (clientId.isNullOrEmpty()) {
                                showAlert("client id 不可为空")
                            } else {
                                IFlyHome.getDeviceCode(clientId, deviceId, responseCallback(dialog))
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.get_client_info) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("client_id", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        dialog.dismiss()
                        val clientId = map["client_id"]
                        if (clientId.isNullOrEmpty()) {
                            showAlert("client id 不可为空")
                        } else {
                            IFlyHome.getClientInfo(clientId, responseCallback(dialog))
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.get_supported_client_info) -> {
                result = IFlyHome.getClientInfos(responseCallback())
            }
            getString(R.string.set_alias) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("alias", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            dialog.dismiss()
                            val alias = map["alias"]
                            if (alias.isNullOrEmpty()) {
                                showAlert("别名不可为空")
                            } else {
                                IFlyHome.updateDeviceAlias(
                                    deviceId,
                                    alias,
                                    responseCallback(dialog)
                                )
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.set_device_mode) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("childrenMode", "")
                    params.putString("continousMode", "")
                    params.putString("zone", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            dialog.dismiss()
                            val childrenMode = map["childrenMode"]
                            val continousMode = map["continousMode"]
                            val zone = map["zone"]
                            if (!childrenMode.isNullOrEmpty() && (childrenMode != "true" && childrenMode != "false")) {
                                showAlert("childrenMode 只能为 true 或者 false")
                            } else if (!continousMode.isNullOrEmpty() && (continousMode != "true" && continousMode != "false")) {
                                showAlert("continousMode 只能为 true 或者 false")
                            } else {
                                val boolChildrenMode =
                                    if (childrenMode.isNullOrEmpty())
                                        null
                                    else
                                        childrenMode == "true"
                                val boolContinousMode =
                                    if (continousMode.isNullOrEmpty())
                                        null
                                    else
                                        continousMode == "true"
                                IFlyHome.updateDeviceInfo(
                                    deviceId,
                                    null,
                                    boolChildrenMode,
                                    boolContinousMode,
                                    zone,
                                    responseCallback(dialog)
                                )
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.get_user_info) -> {
                result = IFlyHome.getUserInfo(responseCallback())
            }
            getString(R.string.get_auth_token) -> {
                AlertDialog.Builder(this)
                    .setMessage("Token: ${IFlyHome.getToken()}")
                    .setPositiveButton(android.R.string.yes, null)
                    .setNegativeButton(android.R.string.copy) { _, _ ->
                        val cm = getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                        cm.primaryClip = ClipData.newPlainText("Token", IFlyHome.getToken())
                    }
                    .show()
            }
            getString(R.string.get_accounts) -> {
                result = IFlyHome.getAccounts(responseCallback())
            }
            getString(R.string.get_web_accounts) -> {
                result = IFlyHome.getWebAccounts(responseCallback())
            }
            getString(R.string.get_play_list) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    result = IFlyHome.getPlayList(deviceId, responseCallback())
                }
            }
            getString(R.string.get_record_tags) -> {
                result = IFlyHome.getRecordTags(responseCallback())
            }
            getString(R.string.get_latest_records) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("tagId", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            dialog.dismiss()
                            val tagId = map["tagId"]
                            if (tagId.isNullOrEmpty()) {
                                showAlert("tagId 不可为空")
                            } else if (!tagId.matches(Regex("\\d+"))) {
                                showAlert("tagId 必须是数字")
                            } else {
                                IFlyHome.loadPlaybackRecord(deviceId, tagId.toInt(), responseCallback())
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.delete_record) -> {
                val dialog = RequestDialog()
                val params = Bundle()
                params.putString("recordId", "")
                dialog.arguments = params
                dialog.handler = object : RequestDialog.RequestHandler {
                    override fun handle(map: Map<String, String>) {
                        dialog.dismiss()
                        val recordId = map["recordId"]
                        if (recordId.isNullOrEmpty()) {
                            showAlert("recordId 不可为空")
                        } else if (!recordId.matches(Regex("\\d+"))) {
                            showAlert("recordId 必须是数字")
                        } else {
                            //为了方便这里demo只填一个id，接口是可以删除多条，或者一条记录的
                            val ids = mutableListOf<Int>()
                            ids.add(recordId.toInt())
                            result = IFlyHome.deleteRecord(ids, responseCallback())
                        }
                    }
                }
                dialog.show(supportFragmentManager, "请求")
            }
            getString(R.string.play_record) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("tagId", "")
                    params.putString("mediaId", "")
                    params.putString("sourceType", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            dialog.dismiss()
                            val tagId = map["tagId"]
                            val mediaId = map["mediaId"]
                            val sourceType = map["sourceType"]
                            when {
                                mediaId.isNullOrEmpty() -> {
                                    showAlert("mediaId 不能为空")
                                    dialog.restoreProgress()
                                }
                                sourceType.isNullOrEmpty() -> {
                                    showAlert("sourceType 不能为空")
                                    dialog.restoreProgress()
                                }
                                tagId.isNullOrEmpty() -> {
                                    showAlert("business 不能为空")
                                    dialog.restoreProgress()
                                }
                                else -> {
                                    if (!tagId.matches(Regex("\\d+"))) {
                                        showAlert("tagId 必须是数字")
                                    } else {
                                        result = IFlyHome.playRecord(deviceId, tagId.toInt(), mediaId, sourceType, responseCallback())
                                        dialog.dismiss()
                                    }
                                }
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.play_album) -> {
                val deviceId = device_id.text.toString()
                if (deviceId.isEmpty()) {
                    showAlert("deviceId 不可为空")
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("albumId", "")
                    params.putString("mediaId", "")
                    params.putString("sourceType", "")
                    params.putString("business", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            dialog.dismiss()
                            val albumId = map["albumId"]
                            val mediaId = map["mediaId"]
                            val sourceType = map["sourceType"]
                            val business = map["business"]
                            when {
                                mediaId.isNullOrEmpty() -> {
                                    showAlert("mediaId 不能为空")
                                    dialog.restoreProgress()
                                }
                                sourceType.isNullOrEmpty() -> {
                                    showAlert("sourceType 不能为空")
                                    dialog.restoreProgress()
                                }
                                albumId.isNullOrEmpty() -> {
                                    showAlert("albumId 不能为空")
                                    dialog.restoreProgress()
                                }
                                business.isNullOrEmpty() -> {
                                    showAlert("business 不能为空")
                                    dialog.restoreProgress()
                                }
                                else -> {
                                    result = IFlyHome.playAlbum(deviceId, albumId, mediaId, sourceType, business, responseCallback())
                                }
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
            getString(R.string.training_plan_api) -> {
                startActivity(Intent(this, IftttActivity::class.java))
            }
        }
        handleResult(result)
    }

    private fun responseCallback(dialog: DialogFragment? = null) = object : ResponseCallback {
        override fun onResponse(response: Response<String>) {
            val intent = Intent(baseContext, ResultActivity::class.java)
            intent.putExtra("code", response.code())
            if (response.isSuccessful) {
                intent.putExtra("result", response.body())
            } else if (response.code() in 400..500) {
                intent.putExtra("result", response.errorBody()?.string())
            }
            progressDialog?.dismiss()
            dialog?.dismiss()
            startActivity(intent)
        }

        override fun onFailure(call: Call<String>, t: Throwable) {
            progressDialog?.dismiss()
            dialog?.dismiss()
            Snackbar.make(container, "请求失败", Snackbar.LENGTH_SHORT).show()
        }
    }

    private fun handleResult(result: Int) {
        when (result) {
            IFlyHome.RESULT_UNINITIALIZED -> {
                showAlert("SDK 未初始化")
            }
            IFlyHome.RESULT_UN_LOGIN -> {
                showAlert("未登录")
            }
            IFlyHome.RESULT_NO_WEB_VIEW -> {
                showAlert("WebView 未注册成功")
            }
            IFlyHome.RESULT_ILLEGAL_PARAMS -> {
                showAlert("参数不合法")
            }
        }
        if (progressDialog?.isShowing == true)
            progressDialog?.dismiss()
    }

    private fun showAlert(message: String) {
        AlertDialog.Builder(this)
            .setMessage(message)
            .setPositiveButton(android.R.string.yes, null)
            .show()
    }

    class ApiViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val tvTitle: TextView = itemView.findViewById(R.id.text1)
    }
}