package com.iflytek.cyber.home.sdk.demo

import android.bluetooth.BluetoothDevice
import com.google.gson.Gson
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.ResponseCallback
import retrofit2.Call
import retrofit2.Response

/**
 * 设备信息缓存类
 */
object BleDeviceStorage {
    data class DeviceInfo(val device: BluetoothDevice, val clientId: String)

    private val deviceInfoSet = HashSet<DeviceInfo>()

    private val clientInfoCache = HashMap<String, ClientInfo>()

    private val callbacks = HashSet<Callback>()

    private val requestingDeviceInfos = HashSet<DeviceInfo>()

    fun addCallback(callback: Callback) {
        callbacks.add(callback)
    }

    fun removeCallback(callback: Callback) {
        callbacks.remove(callback)
    }

    fun getDeviceInfos() = deviceInfoSet

    fun addDevice(device: BluetoothDevice, clientId: String) {
        var requesting = false
        val deviceInfo = DeviceInfo(device, clientId)
        requestingDeviceInfos.map {
            if (it == deviceInfo)
                requesting = true
        }
        if (requesting) {
            return
        }

        var requested = false
        deviceInfoSet.map {
            if (it == deviceInfo)
                requested = true
        }
        if (requested) {
            return
        }

        clientInfoCache[clientId]?.let {
            if (!deviceInfoSet.contains(deviceInfo))
                deviceInfoSet.add(deviceInfo)
            callbacks.map { it.onDeviceScanned(deviceInfoSet) }
        } ?: run {
            requestingDeviceInfos.add(deviceInfo)
            IFlyHome.getClientInfo(clientId, object : ResponseCallback {
                override fun onResponse(response: Response<String>) {
                    if (response.isSuccessful) {
                        val clientInfo = Gson().fromJson<ClientInfo>(response.body(), ClientInfo::class.java)
                        clientInfoCache[clientId] = clientInfo
                        deviceInfoSet.add(deviceInfo)
                        callbacks.map { it.onDeviceScanned(deviceInfoSet) }
                    }
                    requestingDeviceInfos.remove(deviceInfo)
                }

                override fun onFailure(call: Call<String>, t: Throwable) {
                    requestingDeviceInfos.remove(deviceInfo)
                }
            })
        }
    }

    fun getClientInfo(clientId: String, onReceived: (ClientInfo) -> Unit) {
        clientInfoCache[clientId]?.let {
            onReceived.invoke(it)
        } ?: run {
            IFlyHome.getClientInfo(clientId, object : ResponseCallback {
                override fun onResponse(response: Response<String>) {
                    if (response.isSuccessful) {
                        val clientInfo = Gson().fromJson<ClientInfo>(response.body(), ClientInfo::class.java)
                        onReceived.invoke(clientInfo)
                    }
                }

                override fun onFailure(call: Call<String>, t: Throwable) {
                }
            })
        }
    }

    fun clear(notify: Boolean = true) {
        deviceInfoSet.clear()

        if (notify)
            callbacks.map { it.onDeviceCleared() }
    }

    interface Callback {
        fun onDeviceScanned(deviceInfos: HashSet<DeviceInfo>)
        fun onDeviceCleared()
    }
}