package com.iflytek.cyber.home.sdk.demo

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.location.LocationManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.IBinder
import android.provider.Settings
import android.util.Log
import android.view.*
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.PermissionChecker
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.iflytek.home.sdk.IFlyHome
import kotlinx.android.synthetic.main.activity_ble_scan.*

class BleScanActivity : AppCompatActivity(), BleDeviceStorage.Callback {
    private val matchedAdapter = MatchedAdapter()
    private var bleService: BleService? = null

    private var dialog: AlertDialog? = null
    private val adapter = PermissionAdapter()

    private var shouldShowRationale = false

    companion object {
        private const val REQUEST_CODE_LOCATION_SETTINGS = 10011
        private const val REQUEST_ENABLE_BT = 10012
        private const val REQUEST_PERMISSION_LOCATION_IN_SETTINGS = 10013
        private const val REQUEST_PERMISSION_LOCATION = 10014
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility = window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_ble_scan)

        supportActionBar?.run {
            setHomeButtonEnabled(true)
            setDisplayHomeAsUpEnabled(true)
        }

        matched_devices.adapter = matchedAdapter
        matched_devices.layoutManager = LinearLayoutManager(this)

        matchedAdapter.onItemClickListener = object : OnItemClickListener {
            override fun onItemClick(parent: ViewGroup, itemView: View, position: Int) {
                val intent = Intent(this@BleScanActivity, HandleBleDeviceActivity::class.java)
                intent.putExtra("device", matchedAdapter.deviceInfos[position].device)
                intent.putExtra("client_id", matchedAdapter.deviceInfos[position].clientId)
                startActivity(intent)
            }
        }

        bindService(Intent(this, BleService::class.java), serviceConnection, Context.BIND_AUTO_CREATE)

        toggle_scan_state.setOnClickListener {
            bleService?.let { service ->
                if (service.isScanning) {
                    stopScan()
                } else {
                    startScan()
                }
            } ?: run {
                Toast.makeText(this, "请等待服务初始化", Toast.LENGTH_SHORT).show()
            }
        }

        if (!(IFlyHome.isReady() && IFlyHome.isLogin())) {
            Toast.makeText(this, "请等待服务初始化", Toast.LENGTH_SHORT).show()
            finish()
        }
    }

    override fun onResume() {
        super.onResume()

        if (hasReady()) {

        }
        dialog?.let {
            if (hasReady()) {
                it.dismiss()
            } else {
                adapter.updatePermission(generatePermissionArray())
            }
        } ?: run {
            if (!hasReady()) {
                adapter.updatePermission(generatePermissionArray())
                showDialog()
            }
        }
    }

    override fun onPause() {
        super.onPause()

        stopScan()
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home)
            onBackPressed()
        return super.onOptionsItemSelected(item)
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        if (requestCode == REQUEST_PERMISSION_LOCATION) {
            if (PermissionChecker.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                    == PermissionChecker.PERMISSION_GRANTED) {
                if (hasReady()) {
                    bleService?.startScan()
                    dialog?.dismiss()
                } else
                    adapter.updatePermission(generatePermissionArray())
            } else {
                val newShouldShowRationale = ActivityCompat.shouldShowRequestPermissionRationale(
                        this, Manifest.permission.ACCESS_FINE_LOCATION)
                if (shouldShowRationale) {
                    if (newShouldShowRationale) {
                        // 上一次拒绝，且这一次也拒绝
                    } else {
                        // 上一次拒绝，这次又拒绝，且选择不在询问
                    }
                } else {
                    if (newShouldShowRationale) {
                        // 第一次申请
                    } else {
                        // 被永久拒绝
                        requestLocationPermissionInSettings()
                    }
                }
                shouldShowRationale = newShouldShowRationale
            }
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == REQUEST_ENABLE_BT) {
            if (resultCode == Activity.RESULT_CANCELED) {
                finish()
            } else if (resultCode == RESULT_OK) {
                val result = PermissionChecker.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                if (result == PermissionChecker.PERMISSION_GRANTED) {
                    if (hasReady()) {
                        bleService?.startScan()
                        dialog?.dismiss()
                    } else {
                        adapter.updatePermission(generatePermissionArray())
                    }
                }
            }
        } else if (requestCode == REQUEST_PERMISSION_LOCATION_IN_SETTINGS) {
            if (PermissionChecker.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                    == PermissionChecker.PERMISSION_GRANTED) {
                if (hasReady()) {
                    bleService?.startScan()
                    dialog?.dismiss()
                } else
                    adapter.updatePermission(generatePermissionArray())
            } else {

            }
        }
        super.onActivityResult(requestCode, resultCode, data)
    }

    private fun generatePermissionArray(): ArrayList<PermissionModel> {
        val list = ArrayList<PermissionModel>()

        val btOn = PermissionModel(R.drawable.ic_bluetooth_black_24dp,
                getString(R.string.bluetooth), BluetoothAdapter.getDefaultAdapter().isEnabled,
                View.OnClickListener {
                    requestTurnOnBluetooth()
                })
        list.add(btOn)

        val locationPermission = PermissionModel(R.drawable.ic_location_on_black_24dp,
                getString(R.string.location_permission), PermissionChecker.checkSelfPermission(
                this, Manifest.permission.ACCESS_FINE_LOCATION) == PermissionChecker.PERMISSION_GRANTED,
                View.OnClickListener {
                    shouldShowRationale = ActivityCompat.shouldShowRequestPermissionRationale(
                            this, Manifest.permission.ACCESS_FINE_LOCATION)
                    ActivityCompat.requestPermissions(
                            this,
                            arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                            REQUEST_PERMISSION_LOCATION)
                })
        list.add(locationPermission)

        val locationOn = PermissionModel(R.drawable.ic_location_on_black_24dp,
                getString(R.string.location_service), isLocationEnable(this),
                View.OnClickListener {
                    setLocationService()
                })
        list.add(locationOn)

        return list
    }

    private fun showDialog() {

        val dialogBuilder = AlertDialog.Builder(this)

        @SuppressLint("InflateParams")
        val contentView = LayoutInflater.from(this).inflate(R.layout.dialog_multi_permission, null)

        val recyclerView: RecyclerView = contentView.findViewById(R.id.recycler_view)

        recyclerView.adapter = adapter

        val titleView: TextView = contentView.findViewById(R.id.title)
        titleView.setText(R.string.turn_on_these_option_to_add_device)

        contentView.findViewById<View>(R.id.close).setOnClickListener {
            dialog?.dismiss()
            finish()
        }

        dialogBuilder.setView(contentView)
        dialogBuilder.setCancelable(false)

        val dialog = dialogBuilder.create()
        dialog?.window?.run {
            requestFeature(Window.FEATURE_NO_TITLE)
            setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        } ?: run {
            Log.w("Dialog", "dialog window not init")
        }
        dialog?.setOnDismissListener {
            this.dialog = null

            if (!hasReady()) {
                finish()
            }
        }
        dialog.show()
        this.dialog = dialog
    }

    /**
     * 判断 BLE 搜索是否可用
     */
    private fun hasReady(): Boolean {
        return BluetoothAdapter.getDefaultAdapter().isEnabled &&
                isLocationEnable(this) && PermissionChecker.checkSelfPermission(
                this, Manifest.permission.ACCESS_FINE_LOCATION) == PermissionChecker.PERMISSION_GRANTED
    }

    /**
     * Location service if enable
     *
     * @param context context
     * @return location is enable if return true, otherwise disable.
     */
    private fun isLocationEnable(context: Context): Boolean {
        val locationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
        val networkProvider = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
        val gpsProvider = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
        return networkProvider || gpsProvider
    }

    private fun requestTurnOnBluetooth() {
        val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
        startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT)
    }

    private fun requestLocationPermissionInSettings() {
        val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
        val uri = Uri.fromParts("package", applicationContext.packageName, null)
        intent.data = uri
        startActivityForResult(intent, REQUEST_PERMISSION_LOCATION_IN_SETTINGS)
    }

    private fun setLocationService() {
        val locationIntent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
        this.startActivityForResult(locationIntent, REQUEST_CODE_LOCATION_SETTINGS)
    }

    private val serviceConnection = object : ServiceConnection {
        override fun onServiceDisconnected(name: ComponentName?) {
            bleService = null
        }

        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            if (service is BleService.BleServiceBinder) {
                bleService = service.service
            }
            if (hasReady())
                startScan()
        }
    }

    private fun stopScan() {
        if (matchedAdapter.deviceInfos.isEmpty()) {
            status.setText(R.string.no_scanned_device)
        } else {
            status.text = ""
        }
        bleService?.stopScan()

        BleDeviceStorage.clear(false)

        toggle_scan_state.setText(R.string.start_scan)
    }

    private fun startScan() {
        status.setText(R.string.scanning)
        matchedAdapter.deviceInfos.clear()
        bleService?.startScan()
        toggle_scan_state.setText(R.string.stop_scan)
    }

    override fun onStart() {
        super.onStart()

        BleDeviceStorage.addCallback(this)
    }

    override fun onStop() {
        super.onStop()

        BleDeviceStorage.removeCallback(this)
    }

    override fun onDestroy() {
        super.onDestroy()

        unbindService(serviceConnection)
    }

    override fun onDeviceScanned(deviceInfos: HashSet<BleDeviceStorage.DeviceInfo>) {
        matchedAdapter.deviceInfos.clear()
        matchedAdapter.deviceInfos.addAll(deviceInfos)
        matchedAdapter.notifyDataSetChanged()
        if (matchedAdapter.deviceInfos.isEmpty()) {
            status.setText(R.string.no_scanned_device)
        } else {
            status.text = ""
        }
    }

    override fun onDeviceCleared() {
        matchedAdapter.deviceInfos.clear()
        matchedAdapter.notifyDataSetChanged()
        status.setText(R.string.no_scanned_device)
    }


    private class MatchedAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        var onItemClickListener: OnItemClickListener? = null
        var deviceInfos = ArrayList<BleDeviceStorage.DeviceInfo>()
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = MatchedViewHolder(LayoutInflater.from(parent.context).inflate(R.layout.item_matched_device, parent, false))
            holder.itemView.setOnClickListener {
                onItemClickListener?.onItemClick(parent, it, holder.adapterPosition)
            }
            return holder
        }

        override fun getItemCount(): Int {
            return deviceInfos.size
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            if (holder is MatchedViewHolder) {
                val context = holder.itemView.context
                BleDeviceStorage.getClientInfo(deviceInfos[position].clientId) { clientInfo ->
                    Glide.with(context)
                            .load(clientInfo.clientImage)
                            .into(holder.clientImageView)

                    holder.clientIdView.text = clientInfo.clientId
                    holder.clientNameView.text = clientInfo.clientName
                }
            }
        }

    }

    class MatchedViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val clientImageView: ImageView = itemView.findViewById(R.id.client_image)
        val clientNameView: TextView = itemView.findViewById(R.id.client_name)
        val clientIdView: TextView = itemView.findViewById(R.id.client_id)
    }

    interface OnItemClickListener {
        fun onItemClick(parent: ViewGroup, itemView: View, position: Int)
    }


    private class PermissionAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        private val array = ArrayList<PermissionModel>()

        override fun onCreateViewHolder(container: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = PermissionViewHolder(LayoutInflater.from(container.context).inflate(R.layout.item_permission, container, false))
            holder.button.setOnClickListener {
                val position = holder.adapterPosition
                array[position].onClickListener.onClick(it)
            }
            return holder
        }

        fun updatePermission(list: List<PermissionModel>) {
            array.clear()
            array.addAll(list)

            notifyDataSetChanged()
        }

        override fun getItemCount(): Int {
            return array.size
        }

        override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
            if (holder is PermissionViewHolder) {
                val permission = array[position]
                holder.icon.setImageResource(permission.iconResId)
                holder.message.text = permission.message

                if (permission.isEnabled) {
                    holder.tvButton.text = holder.itemView.resources.getString(R.string.already_opened)
                    holder.tvButton.textSize = 12f
                } else {
                    holder.tvButton.text = holder.itemView.resources.getString(R.string.open)
                    holder.tvButton.textSize = 14f
                }
            }
        }
    }

    private class PermissionViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val message: TextView = itemView.findViewById(R.id.message)
        val icon: ImageView = itemView.findViewById(R.id.icon)
        val button: FrameLayout = itemView.findViewById(R.id.button)
        val tvButton: TextView = itemView.findViewById(R.id.tv_button)
    }

    private data class PermissionModel(val iconResId: Int, val message: String, val isEnabled: Boolean, val onClickListener: View.OnClickListener)

}