package com.iflytek.cyber.home.sdk.demo

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import com.google.android.material.snackbar.Snackbar
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.ResponseCallback
import kotlinx.android.synthetic.main.activity_ifttt.*
import retrofit2.Call
import retrofit2.Response

class IftttActivity : AppCompatActivity(), View.OnClickListener {

    @SuppressLint("SetTextI18n")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_ifttt)

        btn_training_list.setOnClickListener(this)
        btn_plan_detail.setOnClickListener(this)
        btn_delete_plan.setOnClickListener(this)
        btn_add_plan.setOnClickListener(this)
        btn_edit_plan.setOnClickListener(this)
        btn_get_actiontypes.setOnClickListener(this)
        btn_get_actiontype.setOnClickListener(this)
        btn_moreitem.setOnClickListener(this)
        btn_check.setOnClickListener(this)

        edt_add_plan_body.setText("{\n" +
                "    \"execType\": \"voice\",\n" +
                "    \"execData\": {\n" +
                "        \"wakeUpWord\": [\n" +
                "            \"今天好吗\",\n" +
                "            \"天气好吗\"\n" +
                "        ]\n" +
                "    },\n" +
                "    \"operations\": [\n" +
                "        {\n" +
                "            \"date\": \"今天\",\n" +
                "            \"city\": \"广东省|广州市\",\n" +
                "            \"name\": \"播报天气\",\n" +
                "            \"alias\": \"当前唤醒设备\",\n" +
                "            \"clientDeviceId\": \"-1\",\n" +
                "            \"text\": \"广东省广州市今天的天气\",\n" +
                "            \"type\": \"weather\"\n" +
                "        }\n" +
                "    ]\n" +
                "}")

        edt_edit_plan_body.setText("{\n" +
                "    \"execType\": \"voice\",\n" +
                "    \"execData\": {\n" +
                "          \"wakeUpWord\": [\n" +
                "            \"今天好吗\",\n" +
                "            \"天气好吗\"\n" +
                "        ]\n" +
                "    },\n" +
                "    \"operations\": [\n" +
                "        {\n" +
                "            \"date\": \"今天\",\n" +
                "            \"city\": \"广东省|广州市\",\n" +
                "            \"name\": \"播报天气\",\n" +
                "            \"alias\": \"当前唤醒设备\",\n" +
                "            \"clientDeviceId\": \"-1\",\n" +
                "            \"text\": \"广东省广州市今天的天气\",\n" +
                "            \"type\": \"weather\"\n" +
                "        },\n" +
                "         {\n" +
                "            \"name\": \"播放音乐\",\n" +
                "            \"alias\": \"当前唤醒设备\",\n" +
                "            \"clientDeviceId\": \"-1\",\n" +
                "            \"text\": \"播放摇滚的歌\",\n" +
                "            \"type\": \"music\",\n" +
                "            \"hobby\": \"场景|摇滚\"\n" +
                "        }\n" +
                "    ]\n" +
                "}")
    }

    private fun responseCallback() = object : ResponseCallback {
        override fun onResponse(response: Response<String>) {
            val intent = Intent(baseContext, ResultActivity::class.java)
            intent.putExtra("code", response.code())
            Log.e("ifttt", "response code: " + response.code() + "  response body: " + response.body())
            if (response.isSuccessful) {
                intent.putExtra("result", response.body())
            } else if (response.code() in 400..500) {
                intent.putExtra("result", response.errorBody()?.string())
            }
            startActivity(intent)
        }

        override fun onFailure(call: Call<String>, t: Throwable) {
            Snackbar.make(container, "请求失败", Snackbar.LENGTH_SHORT).show()
        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.btn_training_list -> {
                IFlyHome.getTrainingPlanList(responseCallback())
            }
            R.id.btn_plan_detail -> {
                val token = edt_token.text.toString()
                if (token.isEmpty()) {
                    Snackbar.make(container, "训练计划token不可为空", Snackbar.LENGTH_SHORT).show()
                    return
                }
                IFlyHome.getPlanDetail(token, responseCallback())
            }
            R.id.btn_delete_plan -> {
                val token = edt_token.text.toString()
                if (token.isEmpty()) {
                    Snackbar.make(container, "训练计划token不可为空", Snackbar.LENGTH_SHORT).show()
                    return
                }
                //可传入多个token
                IFlyHome.deletePlan(arrayListOf(token), responseCallback())
            }
            R.id.btn_add_plan -> {
                val body = edt_add_plan_body.text.toString()
                IFlyHome.addTrainingPlan(null, body, responseCallback())
            }
            R.id.btn_edit_plan -> {
                val token = edt_token.text.toString()
                if (token.isEmpty()) {
                    Snackbar.make(container, "训练计划token不可为空", Snackbar.LENGTH_SHORT).show()
                    return
                }
                val body = edt_edit_plan_body.text.toString()
                IFlyHome.addTrainingPlan(token, body, responseCallback())
            }
            R.id.btn_get_actiontypes -> {
                val execType = edt_exectype.text.toString()
                if (execType.isEmpty()) {
                    Snackbar.make(container, "execType 不可为空，请看接口说明", Snackbar.LENGTH_SHORT).show()
                    return
                }
                IFlyHome.getPlanActionTypes(execType, responseCallback())
            }
            R.id.btn_get_actiontype -> {
                val actionType = edt_actiontype.text.toString()
                if (actionType.isEmpty()) {
                    Snackbar.make(container, "actionType 不可为空，actionType可在 getPlanActionTypes 接口获取", Snackbar.LENGTH_SHORT).show()
                    return
                }
                val execType = edt_exectype.text.toString()
                if (execType.isEmpty()) {
                    Snackbar.make(container, "execType 不可为空，请看接口说明", Snackbar.LENGTH_SHORT).show()
                    return
                }
                //actionType可在 getPlanActionTypes 接口获取
                IFlyHome.getPlanActionType(execType, actionType, responseCallback())
            }
            R.id.btn_moreitem -> {
                val actionType = edt_actiontype.text.toString()
                if (actionType.isEmpty()) {
                    Snackbar.make(container, "actionType 不可为空，actionType可在 getPlanActionTypes 接口获取", Snackbar.LENGTH_SHORT).show()
                    return
                }
                val execType = edt_exectype.text.toString()
                if (execType.isEmpty()) {
                    Snackbar.make(container, "execType 不可为空，请看接口说明", Snackbar.LENGTH_SHORT).show()
                    return
                }
                val iotId = edt_iotid.text.toString()
                if (iotId.isEmpty()) {
                    Snackbar.make(container, "iotId 不可为空, 可在 getPlanActionType 获取", Snackbar.LENGTH_SHORT).show()
                    return
                }
                val body = edt_moreitem_body.text.toString()
                if (body.isEmpty()) {
                    Snackbar.make(container, "body 不可为空，请看接口说明", Snackbar.LENGTH_SHORT).show()
                    return
                }

                IFlyHome.getPlanMoreItem(execType, actionType, iotId, body, responseCallback())
            }
            R.id.btn_check -> {
                val text = edt_text.text.toString()
                if (text.isEmpty()) {
                    Snackbar.make(container, "唤醒语料不可为空", Snackbar.LENGTH_SHORT).show()
                    return
                }

                IFlyHome.checkPlanValid(text, responseCallback())
            }
        }
    }
}