package com.iflytek.cyber.home.sdk.demo

import android.Manifest
import android.app.ProgressDialog
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.PermissionChecker
import com.google.android.material.snackbar.Snackbar
import com.iflytek.home.sdk.IFlyHome
import com.iflytek.home.sdk.callback.IFlyHomeLoginCallback
import com.iflytek.home.sdk.callback.IFlyHomeLogoutCallback
import com.iflytek.home.sdk.callback.ResponseCallback
import kotlinx.android.synthetic.main.activity_main.*
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Response

class MainActivity : AppCompatActivity() {

    private val indexValue = arrayOf(
        IFlyHome.SKILLS,
        IFlyHome.CONTROLLED_DEVICES,
        IFlyHome.ACCOUNTS,
        IFlyHome.CLOCKS,
        IFlyHome.PERSONAL_SOUNDS,
        IFlyHome.BLUETOOTH,
        IFlyHome.CHECK_UPDATE,
        IFlyHome.SPEAKER,
        IFlyHome.TIME_TO_SLEEP,
        IFlyHome.WAKEUP_WORDS,
        IFlyHome.DIALOGUE,
        IFlyHome.COMMUNICATION_SETTING,
        IFlyHome.MESSAGE,
        IFlyHome.DEVICE_ZONE,
        IFlyHome.TRAINING_PLAN
    )
    private val needDeviceIdIndex = arrayOf(
        IFlyHome.BLUETOOTH, IFlyHome.CHECK_UPDATE,
        IFlyHome.SPEAKER, IFlyHome.TIME_TO_SLEEP, IFlyHome.WAKEUP_WORDS, IFlyHome.DIALOGUE,
        IFlyHome.COMMUNICATION_SETTING, IFlyHome.MESSAGE, IFlyHome.DEVICE_ZONE, IFlyHome.SKILLS
    )
    private lateinit var progressDialog: ProgressDialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility =
                window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_main)

        progressDialog = ProgressDialog(this)

        init.setOnClickListener {
            val appId = app_id.text.toString()
            if (appId.isEmpty()) {
                Snackbar.make(container, "appId 不应为空", Snackbar.LENGTH_SHORT).show()
            } else {
                val loginWay = if (login_way.checkedRadioButtonId == R.id.login_way_standard)
                    IFlyHome.LoginWay.STANDARD
                else
                    IFlyHome.LoginWay.CUSTOM_TOKEN
                IFlyHome.init(this, appId, loginWay)

                tv_init_state.text =
                    if (IFlyHome.isReady())
                        getString(R.string.had_init_which_way, IFlyHome.loginWay)
                    else
                        getString(R.string.un_init)
                open_login.isEnabled =
                    IFlyHome.isReady() && IFlyHome.loginWay == IFlyHome.LoginWay.STANDARD
                set_token.isEnabled =
                    IFlyHome.isReady() && IFlyHome.loginWay == IFlyHome.LoginWay.CUSTOM_TOKEN
            }
            updateLoginStateUi()
        }

        login_way.setOnCheckedChangeListener { _, checkedId ->
            open_login_container.visibility =
                if (checkedId == R.id.login_way_standard)
                    View.VISIBLE
                else View.GONE
            set_token_container.visibility =
                if (checkedId == R.id.login_way_custom_token)
                    View.VISIBLE
                else View.GONE
        }
        set_token.setOnClickListener {
            val token = token.text.toString()
            val result = IFlyHome.setCustomToken(token)
            when (result) {
                IFlyHome.RESULT_UNINITIALIZED -> {
                    Snackbar.make(container, "请先初始化 SDK", Snackbar.LENGTH_SHORT).show()
                }
                IFlyHome.RESULT_ILLEGAL_PARAMS -> {
                    Snackbar.make(container, "登录方式错误", Snackbar.LENGTH_SHORT).show()
                }
                else -> {
                    Log.i("SdkDemo", "Token 设置为: '$token'")
                    updateLoginStateUi()
                    Snackbar.make(container, "Token已设置", Snackbar.LENGTH_SHORT).show()
                }
            }
        }

        open_login.setOnClickListener {
            val result = IFlyHome.openLogin(object : IFlyHomeLoginCallback {
                override fun onLoginFailed(type: Int, error: Throwable?) {
                    Toast.makeText(this@MainActivity, "登录失败", Toast.LENGTH_SHORT).show()
                    Log.w("LoginFailed", "type: $type, error: ${error?.toString()}")
                }

                override fun onLoginSuccess() {
                    Toast.makeText(this@MainActivity, "登录成功", Toast.LENGTH_SHORT).show()
                }

                override fun openNewPage(tag: String): Boolean {
                    val intent = Intent(this@MainActivity, WebViewActivity::class.java)
                    intent.putExtra(WebViewActivity.EXTRA_TAG, tag)
                    startActivity(intent)
                    // 若 return false 网页会直接加载，否则由开发者控制何时何处显示
                    return true
                }

            })
            when (result) {
                IFlyHome.RESULT_UNINITIALIZED -> {
                    Snackbar.make(container, "请先初始化 SDK", Snackbar.LENGTH_SHORT).show()
                }
                IFlyHome.RESULT_ILLEGAL_PARAMS -> {
                    Snackbar.make(container, "登录方式错误", Snackbar.LENGTH_SHORT).show()
                }
                else -> {

                }
            }
        }
        logout.setOnClickListener {
            val result = IFlyHome.logout(object : IFlyHomeLogoutCallback {
                override fun onLogoutFailed(t: Throwable?) {
                    Snackbar.make(container, "注销失败", Snackbar.LENGTH_SHORT).show()

                    updateLoginStateUi()

                    Log.w("LogoutFailed", "throwable: ${t?.toString()}")
                }

                override fun onLogoutSuccess() {
                    Snackbar.make(container, "注销成功", Snackbar.LENGTH_SHORT).show()

                    updateLoginStateUi()
                }

            })
            when (result) {
                IFlyHome.RESULT_UNINITIALIZED -> {
                    Snackbar.make(container, "请先初始化 SDK", Snackbar.LENGTH_SHORT).show()
                }
                IFlyHome.RESULT_UN_LOGIN -> {
                    Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
                }
                IFlyHome.RESULT_OK -> {

                }
            }
        }

        spinner_web_page_index.adapter = ArrayAdapter(
            this, android.R.layout.simple_list_item_1,
            android.R.id.text1, resources.getStringArray(R.array.index_name)
        )
        open_web_page.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                val selectedItem = spinner_web_page_index.selectedItemPosition
                if (selectedItem < indexValue.size) {
                    val pageIndex = indexValue[spinner_web_page_index.selectedItemPosition]
                    if (pageIndex in needDeviceIdIndex) {
                        val dialog = RequestDialog()
                        val params = Bundle()
                        params.putString("deviceId", "")
                        dialog.arguments = params
                        dialog.handler = object : RequestDialog.RequestHandler {
                            override fun handle(map: Map<String, String>) {
                                dialog.dismiss()

                                val deviceId = map["deviceId"]
                                if (deviceId.isNullOrEmpty()) {
                                    return
                                } else {
                                    val intent =
                                        Intent(this@MainActivity, WebViewActivity::class.java)
                                    intent.putExtra(WebViewActivity.EXTRA_TARGET, pageIndex)
                                    intent.putExtra(WebViewActivity.EXTRA_DEVICE_ID, deviceId)
                                    startActivity(intent)
                                }
                            }
                        }
                        dialog.show(supportFragmentManager, "请求")
                    } else {
                        val intent = Intent(this, WebViewActivity::class.java)
                        intent.putExtra(WebViewActivity.EXTRA_TARGET, pageIndex)
                        startActivity(intent)
                    }
                } else {
                    val dialog = RequestDialog()
                    val params = Bundle()
                    params.putString("url", "")
                    dialog.arguments = params
                    dialog.handler = object : RequestDialog.RequestHandler {
                        override fun handle(map: Map<String, String>) {
                            val url = map["url"]
                            dialog.dismiss()
                            if (url.isNullOrEmpty()) {
                                Toast.makeText(this@MainActivity, "url 为空", Toast.LENGTH_SHORT)
                                    .show()
                            } else {
                                val intent = Intent(this@MainActivity, WebViewActivity::class.java)
                                intent.putExtra(WebViewActivity.EXTRA_CUSTOM_URL, url)
                                startActivity(intent)
                            }
                        }
                    }
                    dialog.show(supportFragmentManager, "请求")
                }
            }
        }

        api.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                startActivity(Intent(this, ApiActivity::class.java))
            }
        }

        open_web_accounts.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                val progressDialog = ProgressDialog(this@MainActivity)
                progressDialog.setMessage("请稍候")
                progressDialog.show()

                IFlyHome.getWebAccounts(object : ResponseCallback {
                    override fun onResponse(response: Response<String>) {
                        progressDialog.dismiss()
                        if (response.isSuccessful) {
                            Log.d("Result", response.body())
                            val json = JSONObject(response.body())
                            if (json.has("accounts")) {
                                val array = json.getJSONArray("accounts")
                                val values = Array<String>(array.length()) {
                                    array.getJSONObject(it).optString("bind_type")
                                }
                                val adapter = ArrayAdapter<String>(
                                    this@MainActivity,
                                    android.R.layout.select_dialog_item
                                )
                                for (i in 0 until array.length()) {
                                    val item = array.getJSONObject(i)
                                    adapter.add(item.optString("name"))
                                }
                                val dialogBuilder = AlertDialog.Builder(this@MainActivity)
                                dialogBuilder.setTitle("选择账号类型")
                                dialogBuilder.setPositiveButton(android.R.string.cancel, null)
                                dialogBuilder.setAdapter(
                                    adapter
                                ) { _, position ->
                                    val type = values[position]

                                    val intent =
                                        Intent(this@MainActivity, WebViewActivity::class.java)
                                    intent.putExtra(WebViewActivity.EXTRA_TYPE, type)
                                    startActivity(intent)
                                }
                                dialogBuilder.show()
                            } else {
                                Snackbar.make(container, "请求出错: 无账号信息", Snackbar.LENGTH_SHORT)
                                    .show()
                            }
                        } else {
                            Snackbar.make(
                                container,
                                "请求出错: Code${response.code()}",
                                Snackbar.LENGTH_SHORT
                            ).show()
                        }
                    }

                    override fun onFailure(call: Call<String>, t: Throwable) {
                        progressDialog.dismiss()
                        Snackbar.make(container, "请求出错: $t", Snackbar.LENGTH_SHORT).show()
                    }

                })
            }
        }

        sound_wave_config_net.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                startActivity(Intent(this, SoundWavePlayerActivity::class.java))
            }
        }

        ap_config_net.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                startActivity(Intent(this, ApConfigNetActivity::class.java))
            }
        }

        ble_config_net.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                startActivity(Intent(this, BleScanActivity::class.java))
            }
        }
        push_demo.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                startActivity(Intent(this, PushDemoActivity::class.java))
            }
        }

        scan.setOnClickListener {
            if (!IFlyHome.isReady()) {
                Snackbar.make(container, "请先初始化", Snackbar.LENGTH_SHORT).show()
            } else if (!IFlyHome.isLogin()) {
                Snackbar.make(container, "请先登录", Snackbar.LENGTH_SHORT).show()
            } else {
                if (PermissionChecker.checkSelfPermission(this, Manifest.permission.CAMERA)
                    == PermissionChecker.PERMISSION_GRANTED
                )
                    startActivity(Intent(this, ScanActivity::class.java))
                else {
                    ActivityCompat.requestPermissions(
                        this,
                        arrayOf(Manifest.permission.CAMERA),
                        1000
                    )
                }
            }
        }
    }

    override fun onResume() {
        super.onResume()

        login_way.check(
            when (IFlyHome.loginWay) {
                IFlyHome.LoginWay.STANDARD -> R.id.login_way_standard
                IFlyHome.LoginWay.CUSTOM_TOKEN -> R.id.login_way_custom_token
            }
        )
        tv_init_state.text =
            if (IFlyHome.isReady())
                getString(R.string.had_init_which_way, IFlyHome.loginWay)
            else
                getString(R.string.un_init)
        open_login.isEnabled = IFlyHome.isReady()
        set_token.isEnabled = IFlyHome.isReady()
        updateLoginStateUi()
    }

    private fun updateLoginStateUi() {
        val isLogin =
            if (IFlyHome.isReady() && IFlyHome.isLogin())
                getString(R.string.yes)
            else
                getString(R.string.no)
        open_login_is_login.text = getString(R.string.is_login, isLogin)
        custom_token_is_login.text = getString(R.string.is_login, isLogin)
    }
}
