package com.iflytek.cyber.home.sdk.demo

import android.app.Dialog
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.ProgressBar
import android.widget.TextView
import androidx.appcompat.app.AppCompatDialogFragment
import androidx.recyclerview.widget.RecyclerView

class RequestDialog : AppCompatDialogFragment() {
    private var paramsList: RecyclerView? = null
    private val adapter = ParamsAdapter()
    private var progressBar: ProgressBar? = null
    var handler: RequestHandler? = null

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = super.onCreateDialog(savedInstanceState)
        dialog.setTitle("输入请求参数")
        return dialog
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.dialog_request, container, false)
        paramsList = view.findViewById(R.id.params_list)
        progressBar = view.findViewById(R.id.progress_bar)
        view.findViewById<View>(R.id.ensure).setOnClickListener {
            val map = HashMap<String, String>()
            for (i in 0 until (adapter.params?.size ?: 0)) {
                map[adapter.params!![i] ?: ""] = adapter.paramsValue!![i] ?: ""
            }
            progressBar?.visibility = View.VISIBLE
            it.isEnabled = false
            handler?.handle(map)
        }
        view.findViewById<View>(R.id.cancel).setOnClickListener { dismiss() }
        adapter.setParams(arguments!!)
        paramsList?.adapter = adapter
        return view
    }

    fun restoreProgress() {
        progressBar?.visibility = View.GONE
        view?.findViewById<View>(R.id.ensure)?.isEnabled = true
    }

    interface RequestHandler {
        fun handle(map: Map<String, String>)
    }

    inner class ParamsAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
        var params: Array<String?>? = null
        var paramsValue: Array<String?>? = null

        fun setParams(bundle: Bundle) {
            val keySet = bundle.keySet()
            params = arrayOfNulls(keySet.size)
            paramsValue = arrayOfNulls(keySet.size)
            params?.let {
                var i = 0
                keySet.map { key ->
                    it[i++] = key
                }
            }
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
            val holder = ParamsViewHolder(LayoutInflater.from(context).inflate(R.layout.item_params, parent, false))
            holder.editText.addTextChangedListener(object : TextWatcher {
                override fun afterTextChanged(s: Editable?) {

                }

                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
                }

                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                    val index = holder.adapterPosition
                    paramsValue?.let {
                        if (index >= 0 && index < it.size) {
                            it[index] = s?.toString()
                        }
                    }
                }

            })
            return holder
        }

        override fun getItemCount(): Int {
            return params?.size ?: 0
        }

        override fun onBindViewHolder(viewHolder: RecyclerView.ViewHolder, position: Int) {
            if (viewHolder is ParamsViewHolder) {
                params?.let {
                    viewHolder.textView.text = it[position] ?: ""
                }
                paramsValue?.let {
                    viewHolder.editText.setText(it[position] ?: "")
                }
            }
        }

    }

    class ParamsViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val editText: EditText = itemView.findViewById(R.id.edit_text)
        val textView: TextView = itemView.findViewById(R.id.text_view)
    }
}