package com.iflytek.cyber.home.sdk.demo

import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import cn.iflyos.open.library.quiet.SoundWavePlayer
import com.google.android.material.snackbar.Snackbar
import kotlinx.android.synthetic.main.activity_sound_wave_player.*

class SoundWavePlayerActivity : AppCompatActivity() {
    private var player: SoundWavePlayer? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (Build.VERSION.SDK_INT >= 23)
            window.decorView.systemUiVisibility = window.decorView.systemUiVisibility or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        setContentView(R.layout.activity_sound_wave_player)

        supportActionBar?.run {
            setHomeButtonEnabled(true)
            setDisplayHomeAsUpEnabled(true)
        }

        val player = SoundWavePlayer(this)

        play.setOnClickListener {
            if (ssid.text.toString().isEmpty()) {
                Snackbar.make(container, "SSID 不应为空", Snackbar.LENGTH_SHORT).show()
            } else
                player.prepare(re_auth.isChecked, ssid.text.toString(),
                        password.text.toString(), object : SoundWavePlayer.PrepareListener {
                    override fun onPrepared(player: SoundWavePlayer) {
                        player.play()
                    }
                })
        }
        stop.setOnClickListener {
            player.stop()
        }

        this.player = player

        start_auth.setOnClickListener {
            val code = auth_code.text.toString()
            try {
                if (code.length == 6) {
                    val codeNum = code.toInt()
                    val intent = Intent(this, WebViewActivity::class.java)
                    intent.putExtra(WebViewActivity.EXTRA_AUTH_CODE, codeNum)
                    startActivity(intent)
                } else {
                    Snackbar.make(container, "授权码格式不规范", Snackbar.LENGTH_SHORT).show()
                }
            } catch (e: Exception) {
                Snackbar.make(container, "授权码格式不规范", Snackbar.LENGTH_SHORT).show()
                e.printStackTrace()
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem?): Boolean {
        if (item?.itemId == android.R.id.home)
            onBackPressed()
        return super.onOptionsItemSelected(item)
    }

    override fun onPause() {
        super.onPause()

        player?.stop()
    }

    override fun onDestroy() {
        super.onDestroy()
        try {
            player?.destroy()
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
}