package com.iflytek.cyber.home.sdk.demo

import android.content.Context
import android.view.LayoutInflater
import android.widget.TextView
import android.widget.Toast
import androidx.annotation.StringRes


/**
 * 使用给定的 [text] 创建并显示 [Toast]，并防止 MIUI 上前置应用名
 *
 *
 * @param duration Toast 时长, 默认为 [Toast.LENGTH_SHORT]
 */
fun Context.toast(text: CharSequence, duration: Int = Toast.LENGTH_SHORT): Toast {
    val toast = Toast(this)

    val inflate = this.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
    val v = inflate.inflate(R.layout.transient_notification_compat, null)
    val tv = v.findViewById(R.id.message) as TextView
    tv.text = text

    toast.view = v
    toast.duration = duration

    return toast.apply { show() }
}

inline fun Context.toast(@StringRes resId: Int, duration: Int = Toast.LENGTH_SHORT): Toast {
    return Toast.makeText(this, resId, duration).apply { show() }
}